/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include <stdio.h>							    
#include <stdlib.h>
#include "uvspec.h"
#include "../src_c/ascii.h"
#include "../src_c/fortran_and_c.h"
#include "../src_f/f77-uscore.h"



/************************************/
/* prototypes of internal functions */
/************************************/

static int read_aerosol_tau (char* aerosol_filename, int nlyr, int nlambda_t, 
			     float* zd, float** dtau);



/**************************************************************/
/* Setup atmospheric aerosol.                                 */
/**************************************************************/

int setup_aerosol (input_struct input, output_struct *output)
{
  int status=0, lc=0, iv=0;
  int maxawvn = MAXAWVN;

  float *tmp_aerext=NULL, *tmp_aerabs=NULL, *tmp_aersym=NULL, *tmp_aerwvn=NULL;  
  float *tmp_aer_dtau=NULL, *tmp_aer_gg=NULL, *tmp_aer_ssa=NULL;


  void F77_FCN (aerprof) (int* nlyr, float* zd, float* aer_pro, 
			  int* seasn, int* vulcan, float* visib );
  void F77_FCN (aeropt) ( int* vulcan, float* aerext, float* aerabs, 
			  float* aersym, float* aerwvn, float* rh, 
			  int* nlyr, float* zd, int* haze, int* maxawvn ); 
  void F77_FCN (aerwvn) ( float* aerwvn, float* aerabs, float* aersym, float* aerext,
			  float* aer_pro, float* aer_dtau, float* aer_gg, 
			  float* aer_ssa, int* nlyr, float* lambda_t, float* zd, 
			  int* nlambda_t, int* maxawvn );
  void  F77_FCN (scaleaer) (float* aer_dtau, float* lambda_t, int* nlambda_t, 
			    float* alpha, float* beta, int* nlyr);


  if ( input.aer.vulcan > 0 ) {

    /****   Get appropriate aerosol profile and interpolate profile to zd grid.****/

    output->aer.aer_pro  = (float *) calloc(output->atm.nlev, sizeof(float));
    F77_FCN (aerprof) ( &output->atm.nlyr, output->atm.zd, output->aer.aer_pro, 
			&input.aer.seasn, &input.aer.vulcan, &input.aer.visib );

    /****   Get appropriate aerosol optical properties. ****/

    tmp_aerext       = (float *) calloc (maxawvn*output->atm.nlev, sizeof(float));
    tmp_aerabs       = (float *) calloc (maxawvn*output->atm.nlev, sizeof(float));
    tmp_aersym       = (float *) calloc (maxawvn*output->atm.nlev, sizeof(float));
    tmp_aerwvn       = (float *) calloc (maxawvn*output->atm.nlev, sizeof(float));
    F77_FCN (aeropt) ( &input.aer.vulcan, tmp_aerext, tmp_aerabs, 
		       tmp_aersym, tmp_aerwvn, output->atm.rh, 
		       &output->atm.nlyr, output->atm.zd, &input.aer.haze, &maxawvn ); 

    tmp_aer_dtau       = (float *) calloc (output->wl.nlambda_t*output->atm.nlyr, sizeof(float));
    tmp_aer_gg         = (float *) calloc (output->wl.nlambda_t*output->atm.nlyr, sizeof(float));
    tmp_aer_ssa        = (float *) calloc (output->wl.nlambda_t*output->atm.nlyr, sizeof(float));
    F77_FCN (aerwvn) ( tmp_aerwvn, tmp_aerabs, tmp_aersym, tmp_aerext, output->aer.aer_pro, 
			tmp_aer_dtau, tmp_aer_gg, tmp_aer_ssa, &output->atm.nlyr,
			output->wl.lambda_t, output->atm.zd, &output->wl.nlambda_t, &maxawvn );

    output->aer.gg   = fortran2c_2D_float_ary(output->wl.nlambda_t, output->atm.nlyr, tmp_aer_gg );
    output->aer.ssa  = fortran2c_2D_float_ary(output->wl.nlambda_t, output->atm.nlyr, tmp_aer_ssa );
    output->aer.dtau = fortran2c_2D_float_ary(output->wl.nlambda_t, output->atm.nlyr, tmp_aer_dtau );
    
    free(tmp_aer_gg);
    free(tmp_aer_ssa);
    free(tmp_aer_dtau);

    /****   Read in aerosol optical depth file if specified ****/
    if ( strlen(input.aer.tau_filename) > 0 ) {
      status = read_aerosol_tau(input.aer.tau_filename, output->atm.nlyr,
				output->wl.nlambda_t, output->atm.zd,
				output->aer.dtau);
      if ( status != 0.0 ) {
	fprintf (stderr, "Error reading aerosol tau file: %s\n", 
		 input.aer.tau_filename);
	return status;
      }
    }

    /**** Set aerosol asymmetry factor ****/
    if (input.aer.set_gg >= -1.0 ) {
      for (lc=0;lc<output->atm.nlyr;lc++) {
	for (iv=0;iv<output->wl.nlambda_t;iv++) {
	  /*printf("set: %d %d %f %f\n",lc,iv,output->atm.zd[lc],output->aer.ssa[iv][lc]);*/
	  output->aer.gg[iv][lc]  = input.aer.set_gg;
	}
      }
    }
    /**** Set aerosol single scattering albedo ****/
    if (input.aer.set_ssa >= 0.0 ) {
      for (lc=0;lc<output->atm.nlyr;lc++) {
	for (iv=0;iv<output->wl.nlambda_t;iv++) {
	  /*printf("set: %d %d %f %f\n",lc,iv,output->atm.zd[lc],output->aer.ssa[iv][lc]);*/
	  output->aer.ssa[iv][lc]  = input.aer.set_ssa;
	}
      }
    }
    /**** Scale aerosol single scattering albedo ****/
    if (input.aer.scale_ssa > 0.0 ) {
      for (lc=0;lc<output->atm.nlyr;lc++) {
	for (iv=0;iv<output->wl.nlambda_t;iv++) {
	  output->aer.ssa[iv][lc]  = output->aer.ssa[iv][lc] * input.aer.scale_ssa;
	  if (output->aer.ssa[iv][lc]>1.0) {output->aer.ssa[iv][lc] = 1.0;}
	}
      }
    }

    free(tmp_aerext);
    free(tmp_aerabs);
    free(tmp_aersym);
    free(tmp_aerwvn);

    if ( input.aer.beta >= 0.0 ) {
      tmp_aer_dtau  = c2fortran_2D_float_ary(output->wl.nlambda_t, 
					     output->atm.nlyr, output->aer.dtau );
      F77_FCN (scaleaer) (tmp_aer_dtau, output->wl.lambda_t, &output->wl.nlambda_t, 
			  &input.aer.alpha, &input.aer.beta, &output->atm.nlyr);
      output->aer.dtau = fortran2c_2D_float_ary(output->wl.nlambda_t, 
						output->atm.nlyr, tmp_aer_dtau );
      free(tmp_aer_dtau);
    }
  }
  else {
    if ((status = ASCII_calloc_float(&output->aer.dtau, output->wl.nlambda_t, output->atm.nlyr)) != 0 ) return status;
    if ((status = ASCII_calloc_float(&output->aer.gg,   output->wl.nlambda_t, output->atm.nlyr)) != 0 ) return status;
    if ((status = ASCII_calloc_float(&output->aer.ssa,  output->wl.nlambda_t, output->atm.nlyr)) != 0 ) return status;
  }

  return 0;
}


/**************************************************************/
/* Read aerosol profile.                                      */
/**************************************************************/

static int read_aerosol_tau (char* aerosol_filename, int nlyr, int nlambda_t, 
			     float* zd, float** dtau) 
{
  int rows=0;
  int i=0, j=0;
  int status = 0;
  double *aero_zd=NULL, *aero_tau=NULL;


  /* read altitude and optical depth from aerosol file */
  status = read_2c_file (aerosol_filename, 
			 &aero_zd, &aero_tau, &rows);

    
  if (status!=0) {
    switch(status)  {
    case ASCIIFILE_NOT_FOUND: 
      fprintf (stderr, "Aerosol tau file %s not found\n",  aerosol_filename);
      return status;
      break;
    case ASCII_NO_MEMORY: 
      fprintf (stderr, "Not enough memory for reading aerosol tau file %s\n",
	       aerosol_filename);
      return status;
      break;
    case LESS_THAN_TWO_COLUMNS: 
      fprintf (stderr, "Aerosol tau file %s has less than two columns\n",  
	       aerosol_filename);
      return status;
      break;
    default:
      fprintf (stderr, "Error %d reading aerosol file %s\n",  
	       status, aerosol_filename);
      return status;
      break;
    }
  }

  /* check for consistency with atmospheric levels */

  if (rows != nlyr+1) {
    fprintf (stderr, "Aerosol tau file %s: layering differs from atmospheric file\n", 
	     aerosol_filename); 
    status = 3;
    return status;
  }
  
  for (j=0; j<=nlyr; j++) {
    if (zd[j] != aero_zd[j]) {
      fprintf (stderr, "Aerosol layers differ from atmosphere layers %d %f %f\n", 
	       j, aero_zd[j], zd[j]);
      return -1;
    }
  }

  /* copy optical depth to dtau[][] */
  for (i=0;i<nlambda_t;i++)
    for (j=0;j<nlyr;j++)
      dtau[i][j] = aero_tau[j+1];

  /* free memory */
  free (aero_zd);
  free (aero_tau);

  return 0;
}




