/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include <stdio.h>							    
#include <stdlib.h>
#include "uvspec.h"
#include "../src_c/ascii.h"



/************************************/
/* prototypes of internal functions */
/************************************/

static int read_albedo(char* albedo_filename, atm_inp_struct *atm_inp, atm_out_struct *atm_out);



/**************************************************************/
/* Setup surface albedo.                                      */
/**************************************************************/

int setup_albedo (input_struct input, output_struct *output)
{
  int status=0, iv=0;
  int linear=0;

  output->atm.albedo_t = (float *) calloc (output->wl.nlambda_t, sizeof(float));

  if ( input.atm.albedo < 0.0 ) {
    status = read_albedo(input.albedo_filename, &input.atm, &output->atm);
    if ( status != 0.0 ) {
      fprintf (stderr, "Error reading albedo file: %s\n", input.albedo_filename);
      return status;
    }

    /**** Interpolate albedo to transmission grid ****/
    
    linear = 1; /**** Use linear interpolation for the albedo, otherwise 
                      negative albedos or albedos larger than 1 may occurr, 
                      I should know:-) AKy, ****/ 
    status = arb_wvn( output->atm.file_nlambda_albedo, output->atm.file_lambda_albedo,
		      output->atm.file_albedo, 
		      output->wl.nlambda_t, output->wl.lambda_t, output->atm.albedo_t, linear);
    if ( status != 0.0 ) {
      fprintf (stderr, "arb_wvn reports error %d\n", status);
      return status;
    }
    
    /**** Check interpolated albedo values ****/

    for (iv=0;iv<output->wl.nlambda_t;iv++) {
      if ( output->atm.albedo_t[iv] > 1.0 ){
	fprintf (stderr, "Albedo interpolated from file is > 1.0. Wavelength: %f, albedo %f\n", 
		 output->wl.lambda_t[iv], output->atm.albedo_t[iv] );
	fprintf (stderr, "Modify your albedo file: %s\n",input.albedo_filename);
	status=1;
	return status;
      } 
      else 
      if ( output->atm.albedo_t[iv] < 0.0 ){
	fprintf (stderr, "Albedo interpolated from file is < 0.0: %f for wavelength %6.2f\n", 
		 output->atm.albedo_t[iv],output->wl.lambda_t[iv]);
	fprintf (stderr, "Modify your albedo file: %s\n",input.albedo_filename);
	status=2;
	return status;
      } 
    }
  }
  else
    for (iv=0;iv<output->wl.nlambda_t;iv++)
      output->atm.albedo_t[iv] = input.atm.albedo;    
  
  return 0;
}





/**************************************************************/
/* Read albedo from file.                                     */
/**************************************************************/

static int read_albedo(char* albedo_filename,  atm_inp_struct *atm_inp, atm_out_struct *atm_out) 
{
  int rows=0, min_columns=0, max_columns=0;
  int status = 0;
  float **alb_data=NULL;
  
  status = ASCII_file2float (albedo_filename, 
			      &rows, &max_columns, &min_columns, 
			      &alb_data);
  switch(status)  {
  case ASCIIFILE_NOT_FOUND: 
    fprintf (stderr, "Albedo file %s not found\n",  albedo_filename);
    return status;
    break;
  case ASCII_NO_MEMORY: 
    fprintf (stderr, "Not enough memory for reading albedo file %s\n",  albedo_filename);
    return status;
    break;
  case LESS_THAN_TWO_COLUMNS: 
    fprintf (stderr, "Albedo file %s has less than two columns\n",  albedo_filename);
    return status;
    break;
  default:
    break;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, "Inconsistent number of columns in file %s\n", albedo_filename);
    fprintf (stderr, "Columns min = %d, max =%d\n", min_columns, max_columns);
    status = 1;
    return status;
  }  
  if (min_columns<2) {
    fprintf (stderr, "Albedo file %s has less than two columns\n",  albedo_filename);
    status = 2;
    return status;
  }
  
  atm_out->file_nlambda_albedo = rows;

  /*  printf("atm rows %d\n",rows);*/
  
  atm_out->file_albedo = (float *) calloc (atm_out->file_nlambda_albedo, sizeof(float));
  atm_out->file_lambda_albedo = (float *) calloc (atm_out->file_nlambda_albedo, sizeof(float));

  atm_out->file_lambda_albedo = ASCII_column_float(alb_data, atm_out->file_nlambda_albedo, 0 );
  atm_out->file_albedo        = ASCII_column_float(alb_data, atm_out->file_nlambda_albedo, 1 );

  ASCII_free_float(alb_data,rows);

  return 0;
}

