/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include <stdio.h>							    
#include <stdlib.h>
#include "uvspec.h"
#include "../src_c/ascii.h"
#include "../src_c/radiative.h"
#include "../src_f/f77-uscore.h"



/************************************/
/* prototypes of internal functions */
/************************************/

static int read_atmosphere(char* atmosphere_filename, atm_inp_struct *atm_inp, 
			   atm_out_struct *atm_out, int other_column);

static int shift_atmosphere(atm_inp_struct *atm_inp, 
			    atm_out_struct *atm_out);




/**************************************************************/
/* Setup model atmosphere.                                    */
/**************************************************************/

int setup_atmosphere (input_struct input, output_struct *output)
{
  int status=0, lc=0;
  float scale_factor=0;

  float F77_FCN (o3column) (float* dens, int* nlyr, float* zd);
  void  F77_FCN (scaleozone) (float* dens, int* nlyr, 
			      float* ozone_column, float* zd, float* ozone_scale_factor);


  status = read_atmosphere(input.atmosphere_filename, 
			   &input.atm, &output->atm, input.crs.column);

  if (status!=0) {
    fprintf (stderr, " ... error %d reading atmosphere from %s\n",
	     status, input.atmosphere_filename);
    return status;
  }
  
  
  /* Shift atmosphere to given altitude */
  if (input.atm.altitude != 0) {
    status = shift_atmosphere(&input.atm, &output->atm);
    if (status !=0) {
      fprintf (stderr, "error %d shifting atmosphere\n", status);
      return status;
    }
  }

  /* Scale pressure and air profiles */
  if (input.pressure >= 0.0) {
    scale_factor = input.pressure / output->atm.press[output->atm.nlyr];
    for (lc=0;lc<=output->atm.nlyr;lc++) {
      output->atm.press[lc] *= scale_factor;
      output->atm.dens[AIR][lc] *= scale_factor;
    } 
  }

  /**** Scale ozone column ****/
  output->ozone_column = F77_FCN (o3column) (output->atm.dens[O3], &(output->atm.nlyr), output->atm.zd);
  
  if ( input.ozone_column >= 0.0 ) {
    F77_FCN (scaleozone) (output->atm.dens[O3], &output->atm.nlyr, &input.ozone_column,
			  output->atm.zd, &output->ozone_scale_factor);
    output->ozone_column = F77_FCN (o3column) (output->atm.dens[O3], &output->atm.nlyr, output->atm.zd); 
  }

  return 0;
}



/**************************************************************/
/* Setup temperature at output levels.                        */
/**************************************************************/

int setup_temperature (input_struct input, output_struct *output)
{
  int status=0, lev=0;
  int linear=0;
  float *tmp_temper=NULL, *tmp_zd=NULL;

  /* Linearly interpolate output temperature to zout */
  /* sort profile in ascending order, assuming that */
  /* the original profile was descending            */
  
  tmp_zd     = (float *) calloc(output->atm.nlev, sizeof(float));
  tmp_temper = (float *) calloc(output->atm.nlev, sizeof(float));
  
  
  for (lev=0; lev<output->atm.nlev; lev++) {
    tmp_zd[lev]       = output->atm.zd          [output->atm.nlev-1-lev];
    tmp_temper[lev]   = output->atm.temper      [output->atm.nlev-1-lev];
  }

  output->atm.temper_out  = (float *) calloc (input.atm.nzout, sizeof(float));
  linear = 1;
  status = arb_wvn(output->atm.nlev, tmp_zd, tmp_temper, 
		   input.atm.nzout, input.atm.zout, output->atm.temper_out, linear);

  if (status != 0) {
    fprintf (stderr, "Error interpolating temperature to output grid\n");
    return status;
  }

  free(tmp_temper); 
  free(tmp_zd);

  return 0;
}




/**************************************************************/
/* Read atmosphere file and write to atm_out_struct           */
/**************************************************************/

static int read_atmosphere(char* atmosphere_filename, atm_inp_struct *atm_inp, 
			   atm_out_struct *atm_out, int other_column) 
{
  int rows=0, min_columns=0, max_columns=0;
  int status = 0;
  float **atm_data=NULL;
  
  /*  printf("%s\n",input.atmosphere_filename); */

  status = ASCII_file2float (atmosphere_filename, 
			      &rows, &max_columns, &min_columns, 
			      &atm_data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, atmosphere_filename);
    return status;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
    return -1;
  }  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  if (min_columns<other_column) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  /*  printf("atm: %d\n",other_column);*/
  
  atm_out->nlev = rows;
  atm_out->nlyr = rows-1;
  /*  printf("atm rows %d\n",rows);*/
  
  /* allocate memory for density array */
  atm_out->dens = calloc(atm_out->ndens, sizeof(float *));
  if (atm_out->dens == NULL)
    return -1;
  
  atm_out->press     = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_PRESS );
  atm_out->temper    = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_TEMPER );
  atm_out->zd        = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_ZD );

  atm_out->dens[AIR] = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_AIR );
  atm_out->dens[O3]  = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_O3 );
  atm_out->dens[O2]  = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_O2 );
  atm_out->dens[H2O] = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_H2O);
  atm_out->dens[CO2] = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_CO2);
  atm_out->dens[NO2] = ASCII_column_float(atm_data, atm_out->nlev, ATMFILE_NO2);

  atm_out->rh        = (float *) calloc (atm_out->nlev, sizeof(float));

  status = ASCII_free_float(atm_data, rows);
  /*  printf("done read atmosphere\n");*/
  return 0;
}




/**************************************************************/
/* Shift struct atm_out_struct to specified altitude.         */
/**************************************************************/

static int shift_atmosphere(atm_inp_struct *atm_inp, 
			    atm_out_struct *atm_out)
{
  int i=0, lev=0;

  float *z_shift = calloc(atm_out->nlev, sizeof(float));
  float press=0, temper=0, rh=0, air=0, o3=0, o2=0, h2o=0, co2=0, no2=0;
  
  /* calculate shifted levels (altitude now in km) */
  for (i=0; i<atm_out->nlev; i++)
    z_shift[i] = atm_out->zd[i] - atm_inp->altitude;


  /* check if the shifted profile extends down to 0 */
  if (z_shift[atm_out->nlev-1] > 0) {
    fprintf (stderr, "Sorry, the first level in the atmospheric data profile\n");
    fprintf (stderr, "is too high! It must be at least as low as ALTITUDE.\n");
    fprintf (stderr, "Found the first level at %f\n", 
	     atm_out->zd[atm_out->nlev-1]);
   return -1;
  }


  /* determine the first level below 0 */
  lev = atm_out->nlev;
  while (z_shift[--lev]<=0) {
    if (lev<=1) {
      fprintf (stderr, "Sorry, ALTITUDE is higher than the uppermost layer in the\n");
      fprintf (stderr, "atmospheric data profile.\n");
      return -1;
    }
  }

  /* nlev is now the first level above 0 */

  press = atm_out->press[lev] + (0.0-z_shift[lev])*
    (atm_out->press[lev+1]-atm_out->press[lev])/(z_shift[lev+1]-z_shift[lev]);

  temper = atm_out->temper[lev] + (0.0-z_shift[lev])*
    (atm_out->temper[lev+1]-atm_out->temper[lev])/(z_shift[lev+1]-z_shift[lev]);

  rh = atm_out->rh[lev] + (0.0-z_shift[lev])*
    (atm_out->rh[lev+1]-atm_out->rh[lev])/(z_shift[lev+1]-z_shift[lev]);

  air = atm_out->dens[AIR][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[AIR][lev+1]-atm_out->dens[AIR][lev])/(z_shift[lev+1]-z_shift[lev]);

  o3 = atm_out->dens[O3][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[O3][lev+1]-atm_out->dens[O3][lev])/(z_shift[lev+1]-z_shift[lev]);

  o2 = atm_out->dens[O2][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[O2][lev+1]-atm_out->dens[O2][lev])/(z_shift[lev+1]-z_shift[lev]);

  h2o = atm_out->dens[H2O][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[H2O][lev+1]-atm_out->dens[H2O][lev])/(z_shift[lev+1]-z_shift[lev]);

  co2 = atm_out->dens[CO2][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[CO2][lev+1]-atm_out->dens[CO2][lev])/(z_shift[lev+1]-z_shift[lev]);

  no2 = atm_out->dens[NO2][lev] + (0.0-z_shift[lev])*
    (atm_out->dens[NO2][lev+1]-atm_out->dens[NO2][lev])/(z_shift[lev+1]-z_shift[lev]);


  /* number of levels is probably smaller now */
  atm_out->nlev = lev+2;
  atm_out->nlyr = atm_out->nlev-1;

  z_shift[atm_out->nlev-1]=0;

  atm_out->press    [atm_out->nlev-1]  = press;
  atm_out->temper   [atm_out->nlev-1]  = temper;
  atm_out->rh       [atm_out->nlev-1]  = rh;

  atm_out->dens[AIR][atm_out->nlev-1]  = air;
  atm_out->dens[O3] [atm_out->nlev-1]  = o3;
  atm_out->dens[O2] [atm_out->nlev-1]  = o2;
  atm_out->dens[H2O][atm_out->nlev-1]  = h2o;
  atm_out->dens[CO2][atm_out->nlev-1]  = co2;
  atm_out->dens[NO2][atm_out->nlev-1]  = no2;

  for (lev=0; lev<atm_out->nlev; lev++)
    atm_out->zd[lev] = z_shift[lev];    


  return 0;  /* if o.k. */
}




