/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include <stdio.h>							    
#include <stdlib.h>
#include "uvspec.h"
#include "../src_c/ascii.h"
#include "../src_c/sun.h"



/************************************/
/* prototypes of internal functions */
/************************************/

static int read_extraterrestrial(char* extraterrestrial_filename, 
				 wl_inp_struct *wl_inp, wl_out_struct *wl_out,
				 float wl_start, float wl_end);




/**************************************************************/
/* Setup extraterrestrial irradiance.                         */
/**************************************************************/

int setup_extraterrestrial (input_struct input, output_struct *output)
{
  int status=0, iv=0;

  /**** Read extraterrestrial irradiance ****/
  status = read_extraterrestrial(input.extraterrestrial_filename, 
				 &input.wl, &output->wl, 
				 input.wl.wl_start, input.wl.wl_end);
  if (status!=0) {
    fprintf (stderr, " ... error %d reading %s\n",
	     status, input.extraterrestrial_filename);
    return status;
  }

  /**** Correction factor for Earth-Sun distance ****/
  output->eccentricity_factor = 1.0;
  
  if ( input.day_of_year > 0.0 ) {
    output->eccentricity_factor = eccentricity( input.day_of_year );
    for (iv=0;iv<output->wl.nlambda_h;iv++)
      output->wl.fbeam[iv] *= output->eccentricity_factor;
  }

  return 0;
}



/**************************************************************/
/* Read extraterrestrial irradiance.                          */
/**************************************************************/

static int read_extraterrestrial(char* extraterrestrial_filename, 
				 wl_inp_struct *wl_inp, wl_out_struct *wl_out,
				 float wl_start, float wl_end) 
{
  int iv, rows=0, min_columns=0, max_columns=0, nlambda;
  int status = 0;
  float **wl_data=NULL;
  float *tmp_lambda=NULL, *tmp_fbeam=NULL;
  
  /*  printf("%s\n",input.atmosphere_filename); */

  status = ASCII_file2float (extraterrestrial_filename, 
			     &rows, &max_columns, &min_columns, 
			     &wl_data);
  if (status!=0) {
    fprintf (stderr, " ... error %d opening file %s\n", status, extraterrestrial_filename);
    return status;
  }
  if (max_columns!=min_columns) {
    fprintf (stderr, " !! ATTENTION !! Inconsistent number of columns\n");
    fprintf (stderr, "     min = %d, max =%d\n", min_columns, max_columns);
  }  
  if (min_columns<2) {
    fprintf (stderr, " ... ending, too few columns\n");
    return -1;
  }
  
  wl_out->nlambda_h = rows;

  /*  printf("atm rows %d\n",rows);*/
  
  tmp_lambda = ASCII_column_float(wl_data, wl_out->nlambda_h, 0 );
  tmp_fbeam  = ASCII_column_float(wl_data, wl_out->nlambda_h, 1 );

  nlambda = 0;
  for (iv=0;iv<wl_out->nlambda_h;iv++) {
    if ( (tmp_lambda[iv]>=wl_start) && (tmp_lambda[iv]<=wl_end) ) {
      nlambda++;
    }
  }

  wl_out->lambda_h = (float *) calloc (nlambda, sizeof(float));
  wl_out->fbeam    = (float *) calloc (nlambda, sizeof(float));

  nlambda = 0;
  for (iv=0;iv<wl_out->nlambda_h;iv++) {
    if ( (tmp_lambda[iv]>=wl_start) && (tmp_lambda[iv]<=wl_end) ) {
      wl_out->lambda_h[nlambda] = tmp_lambda[iv];
      wl_out->fbeam[nlambda]    = tmp_fbeam[iv];
      nlambda++;
    }
  }

  wl_out->nlambda_h=nlambda;

  /* free memory */
  ASCII_free_float (wl_data, rows);
  free(tmp_lambda);
  free(tmp_fbeam);

  return 0;
}



