/************************************************************************/
/*                                                                      */
/* Copyright (C) 1997 Arve Kylling                                      */
/*                                                                      */
/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 1, or (at your option)  */
/* any later version.                                                   */
/*                                                                      */
/* This program is distributed in the hope that it will be useful,      */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of       */
/* MERCHANTABILITY of FITNESS FOR A PARTICULAR PURPOSE. See the         */
/* GNU General Public License for more details.                         */
/*                                                                      */
/* To obtain a copy of the GNU General Public License write to the      */
/* Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,   */
/* USA.                                                                 */
/*                                                                      */
/************************************************************************/

#include "uvspec.h"

/**************************************************************/
/* The main uvspec function.                                  */
/**************************************************************/

int uvspec (input_struct input, output_struct *output)
{
  int status = 0;


  /**** Check the model input data ****/
  status = uvspec_check(input);
  if (status!=0) {
    fprintf (stderr, "%d error(s) in uvspec input-file, aborting\n", status);
    return status;
  }


  /**** Set wavelength grid for the transmittance calculation ****/
  status = setup_wlgrid (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up wavelength grid\n", status);
    return status;
  }


  /**** Cloudless sky atmosphere ****/
  status = setup_atmosphere (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up atmosphere\n", status);
    return status;
  }
  

  /**** Setup temperature at computing levels ****/
  status = setup_temperature (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up temperature\n", status);
    return status;
  }
  

  /****  Read extraterrestrial spectrum and correct for Earth-Sun distance ****/
  status = setup_extraterrestrial (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up extraterrestrial irradiance\n", status);
    return status;
  }


  /**** Read cross section files, like O3, BrO, OclO, NO2, etc. ****/
  /**** Also calculate Rayleigh scattering cross section        ****/
  status = setup_crs (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up cross sections\n", status);
    return status;
  }


  /**** Setup wavelength dependent albedo ****/
  status = setup_albedo (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up wavelength dependent albedo\n", status);
    return status;
  }
  

  /**** Setup sza and phi0 ****/
  status = setup_sza (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up sza and phi0\n", status);
    return status;
  }
  

  /**** Ice cloud stuff ****/
  status = setup_ice (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up ice cloud properties\n", status);
    return status;
  }


  /**** Water cloud stuff ****/
  status = setup_wcloud (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up water cloud properties\n", status);
    return status;
  }


  /**** Independent pixel stuff ****/
  status = setup_ipa (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up IPA properties\n", status);
    return status;
  }
  

  /****   Aerosol stuff   ****/
  status = setup_aerosol (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d setting up aerosol properties\n", status);
    return status;
  }


  /****     Solve RTE     ****/

  status = solve_rte (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d solving RTE\n", status);
    return status;
  }


  /**** Interpolate transmittance to high resolution wavelength grid ****/
  status = interpolate_transmittance (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d interpolating model output to hires wavelength grid\n", status);
    return status;
  }


  /**** Multiply with extraterrestrial irradiance ****/
  status = multiply_extraterrestrial (input, output);
  if (status!=0) {
    fprintf (stderr, "Error %d multiplying with extraterrestrial irradiance\n", status);
    return status;
  }


  /**** Convolve with slit function ****/
  if (input.convolve) {
    status = convolve (input, output);
    if (status!=0) {
      fprintf (stderr, "Error %d convolving hires spectrum with slit function\n", status);
      return status;
    }
  }


  /**** Finally interpolate to output grid ****/
  if (input.spline) {
    status = spline_interpolate (input, output);
    if (status!=0) {
      fprintf (stderr, "Error %d interpolating data spectrum to output grid\n", status);
      return status;
    }
  }

  return 0;
}



/**************************************************************/
/* Check the model input for consistency.                     */
/**************************************************************/

int uvspec_check (input_struct Input)
{
  int i=0, status=0;
  FILE *f=NULL;

  /* ***FIXME*** Still need to check (also in uvspec function):
     aerosol_haze
     aerosol_scale_ssa
     aerosol_set_gg
     aerosol_set_ssa
     aerosol_season
     aerosol_visibility
     aerosol_vulcan
     angstrom
     atmosphere_file
     data_files_path
     nstr
     rte_solver
     solar_file
     sza
     sza_file
     umu
     wc_file
     wc_set_gg
     wc_set_ssa
     wc_layer
     wvn
     zout
   */

  /* Albedo checks */
  
  if ( Input.atm.albedo > 1.0 ) {
    fprintf (stderr, "Surface albedo > 1.0: %f\n", Input.atm.albedo);
    status--;
  }
  else if ( Input.atm.albedo < 0.0 && strlen(Input.albedo_filename)==0 ) {
    fprintf (stderr, "Surface albedo < 0.0: %f\n", Input.atm.albedo);
    status--;
  }

  /* Day of year checks */

  if ( Input.day_of_year > 366 ) {
    fprintf (stderr, "Day of year > 366: %d\n", Input.day_of_year);
    status--;
  }
  else if ( Input.day_of_year < 0 && Input.day_of_year != -999 ) {
    fprintf (stderr, "Day of year < 0: %d\n", Input.day_of_year);
    status--;
  }

  /* Ozone column checks */

  if ( Input.ozone_column < 0.0 ) {
    fprintf (stderr, "Ozone_column < 0.0: %f\n", Input.ozone_column);
    status--;
  }

  /* RTE checks */

  if ( (Input.rte.solver==SDISORT) && strlen(Input.watercloud_filename)>0 ) {
    fprintf (stderr, "Warning: sdisort is not suited for calculations involving clouds\n");
  }
  else if ( (Input.rte.solver==TWOSTR) && Input.rte.nprndis>0 ) {
    for (i=0;i<Input.rte.nprndis;i++) {
      if ( !(Input.rte.prndis[i] == 1 || Input.rte.prndis[i] == 2) ) { 
	fprintf (stderr, "Twostr can only handle prndis set to 1 and/or 2\n");
	status--;
      }
      else {
	fclose(f);
      }
    }
  }

  /* Check wavelength ranges */
  if ( Input.spline ) {
    if ( Input.wl.wl_start > Input.spline_lambda_0 )  {
      fprintf (stderr, "Spline lambda %f is smaller than wvn lambda %f\n",
	       Input.spline_lambda_0, Input.wl.wl_start);
      status--;
    }
    if ( Input.wl.wl_end < Input.spline_lambda_1 )  {
      fprintf (stderr, "Spline lambda %f is greater than wvn lambda %f\n",
	       Input.spline_lambda_1, Input.wl.wl_end);
      status--;
    }
  }

  /* Check that slit_function file exists */
  if ( Input.convolve ) {
    if ( (f = fopen(Input.slit_function_filename, "r")) == NULL) {
      fprintf (stderr, "Can not read slit function file %s\n", Input.slit_function_filename);
      status--;
    }
    else 
      fclose(f);
  }

  
  /* Cloud checks */
  if ( Input.cloudcover != -999.0 )
    if (Input.cloudcover < 0.0 || Input.cloudcover > 1.0) {
      fprintf (stderr, "wc_cloudcover not between 0 and 1: %f\n", Input.cloudcover);
      status--;
    }

  return status;
}

